//////////////////////////////////////////////////////////////////////////////////
//
// CFGFILE.CPP - Definition of AConfigFile, the configuration file template class
//
// (c) 1997 by Stefan Ruck, Frankfurt/Main, Germany
//
///////////////////////////////////////////////////////////////////////////////////

//**************************************************************************
// AConfigFile :: AConfigFile - Constructor for AConfigFile                *
//**************************************************************************

template <class T>
AConfigFile <T> :: AConfigFile ()
                   : AConfigBase ("CFG"),
                     AFile (&m_ConfigData, sizeof (T))
{
  memset (&m_ConfigData, 0, sizeof (T));
}

//**************************************************************************
// AConfigFile :: ~AConfigFile - Destructor for AConfigFile                *
//**************************************************************************

template <class T>
AConfigFile <T> :: ~AConfigFile ()

{
}

//**************************************************************************
// AConfigFile :: read - reads the configuration                           *
//**************************************************************************

template <class T>
APIRET AConfigFile <T> :: read (const ULONG ulFileAttribute /* = FILE_NORMAL */,
                                const ULONG ulOpenFlag /* = FILE_OPEN */,
                                const ULONG ulOpenMode /* = OPEN_ACCESS_READONLY | OPEN_SHARE_DENYWRITE */,
                                const PEAOP2 pEABuf /* = 0L */)

{
  APIRET ret = NO_ERROR;

  if(!isOpen ())  // is the file open?
     {
      ret = open (ulFileAttribute, ulOpenFlag, ulOpenMode, pEABuf);  // no, open the file
      if(ret != NO_ERROR)                                            // occured an error while opening the file?
         return (ret);                                               // yes, return
     }

  ret = AFile :: read ();       // read the configuration data

  if(ret != NO_ERROR)
     {
      close ();
      return (ret);
     }

  ret = close ();               // always close the file

  return (ret);       
}

//**************************************************************************
// AConfigFile :: write - writes the configuration                         *
//**************************************************************************

template <class T>
APIRET AConfigFile <T> :: write (const ULONG ulFileAttribute /* = FILE_NORMAL */,
                                 const ULONG ulOpenFlag /* = FILE_OPEN | FILE_CREATE */,
                                 const ULONG ulOpenMode /* = OPEN_ACCESS_WRITEONLY | OPEN_SHARE_DENYREADWRITE */,
                                 const PEAOP2 pEABuf /* = 0L */)

{
  APIRET ret = NO_ERROR;

  if(!isOpen ())  // is the file open?
     {
      ret = open (ulFileAttribute, ulOpenFlag, ulOpenMode, pEABuf);  // no, open the file
      if(ret != NO_ERROR)                                            // occured an error while opening the file?
         return (ret);                                               // yes, return
     }

  ret = AFile :: write ();       // write the configuration data

  if(ret != NO_ERROR)
     {
      close ();
      return (ret);
     }

  ret = close ();               // always close the file

  return (ret);       
}
