///////////////////////////////////////////////////////////////////////////////////////////
//
// INITEST.CPP - Description of AIniFileTest, the configuration file sample application
//
// (c) 1997 by Stefan Ruck, Frankfurt/Main, Germany
//
///////////////////////////////////////////////////////////////////////////////////////////

#pragma info (none)

#include <os2.h>
#include <imsgbox.hpp>
#include <istring.hpp>
#include <irect.hpp>

#pragma info (restore)

#include "initest.rch"
#include "initest.hpp"
#include "about.hpp"

//*************************************************************************
// main  - Application entry point                                        *
//*************************************************************************
int main(int argc, char ** argv)
{
  try{
      AIniFileTest mainWindow (WND_MAIN);        // Create our main window
      IApplication :: current (). run ();        // Get the current application and run it
      return (0);
     }
  catch(IException& rException)
        {
         IString str ("Encountered an exception:\n\n");
         if (!rException. textCount ())
             str += "No error text available";
         else
             {
              char szBuffer [21];
              str += rException. text (rException. textCount () - 1);
              str += " Error ID: ";
              str += _ltoa (rException. errorId (), szBuffer, 10);
             }
         IMessageBox msgBox (IWindow :: desktopWindow ());
         msgBox. setTitle (rException. name ());
         msgBox.show(str, IMessageBox::okButton   |
                          IMessageBox::informationIcon  |
                          IMessageBox::applicationModal |
                          IMessageBox::moveable);
         return (0);
        }
}

//*****************************************************************
// AIniFileTest :: AIniFileTest - Constructor of AIniFileTest     *
//*****************************************************************
AIniFileTest :: AIniFileTest(unsigned long windowId)
               : IFrameWindow (IFrameWindow::titleBar
                               | IFrameWindow::systemMenu
                               | IFrameWindow::sizingBorder
                               | IFrameWindow::minimizeButton
                               | IFrameWindow::maximizeButton
                               | IFrameWindow::windowList
                               | IFrameWindow::appDBCSStatus
                               | IFrameWindow::minimizedIcon
                               | IFrameWindow::menuBar
                               | IFrameWindow::accelerator,
                               windowId),
                  m_pMenuBar (0L),
                  m_pCfgConfig (0L),
                  m_pIniConfig (0L)
{
  setExceptionFunction (&excHandler);

  if (!setupVariables ())                         // setup the variables
      {
       cleanUp ();                                // an error during setupVariables occures, so clean up
       ITHROW (IException ("AIniFileTest :: setupVariables failed", 0)); // and throw an exception
      }

  ICommandHandler :: handleEventsFor (this);      // set self as command handler

  setFocus ();                                    // set the focus

  show ();                                        // show the window
}

//*****************************************************************
// AIniFileTest :: ~AIniFileTest - Destructor of AIniFileTest     *
//*****************************************************************
AIniFileTest :: ~AIniFileTest(void)

{
  cleanUp ();
}

//**********************************************************************
// AIniFileTest :: cleanUp - clean up of the allocated resources       *
//**********************************************************************
void AIniFileTest :: cleanUp (void)
{
  delete m_pMenuBar;

  delete m_pCfgConfig;

  delete m_pIniConfig;
}

//*********************************************************************
// AIniFileTest :: setupVariables - variable set up                   * 
//*********************************************************************
Boolean AIniFileTest :: setupVariables (void)
{
  m_pMenuBar = new IMenuBar (WND_MAIN, this);            // initialize the menu bar

  try
     {
      m_pCfgConfig = new ACfgConfig ();                  // initialize the configuration file
      m_pIniConfig = new AIniConfig ();                  // initialize the initialization file
     }
  catch(IAccessError& rException)
        {
         IString str ("Encountered an exception:\n\n");
         if (!rException. textCount ())
             str += "No error text available";
         else
             {
              char szBuffer [21];
              str += rException. text (rException. textCount () - 1);
              str += " Error ID: ";
              str += _ltoa (rException. errorId (), szBuffer, 10);
             }
         IMessageBox msgBox (desktopWindow ());
         msgBox. setTitle (rException. name ());
         msgBox.show(str, IMessageBox::okButton   |
                          IMessageBox::informationIcon  |
                          IMessageBox::applicationModal |
                          IMessageBox::moveable);
         return (false);
        }

  // set the window's size from the configuration file

  m_pCfgConfig -> read ();                           // read the configuration

  if(m_pCfgConfig -> isSizeSaved ())                 // was a size saved
     {                                               // yes
      ISize sSize (m_pCfgConfig -> getSize ());      // get the size
      sizeTo (sSize);                                // set the window's size
     }

  // set the window's position from the initialization file

  if(m_pIniConfig -> isPositionSaved ())                 // was a value for the position saved?
     {                                                   // yes
      IPoint pPosition (m_pIniConfig -> getPosition ()); // get the position
      moveTo (pPosition);                                // set the window's position
     }

  return (true);
}

//*********************************************************************
// AIniFileTest :: command - handles Command-Events                   *
//*********************************************************************
Boolean AIniFileTest :: command(ICommandEvent &evt)
{
  Boolean filtered = true;                                        // event handled

  switch(evt.commandId())                      	                  // handle the menu commands
         {
          case MI_FILE_EXIT:                                      // exit the program
               {
                m_pCfgConfig -> setSize (size (). asSIZEL ());    // set size of configuration
                m_pCfgConfig -> write ();                         // save the configuration
                m_pIniConfig -> setPosition (position (). asPOINTL ()); // set position of ini
                close ();                                         // close the frame window
                break;
               }
          case MI_HELP_ABOUT:                                     // show the product information
               {
                AAbout * pDlgAbout = new AAbout (this);
                pDlgAbout -> showModally ();
                delete pDlgAbout;
                break;
               }
          default:
              filtered = false;                                   // event not handled
              break;
         }

  return filtered;
}

//*************************************************************************
// AIniFileTest :: ExceptionHandler - default rException handling         *
//*************************************************************************
Boolean AIniFileTest :: ExceptionHandler :: handleException(IException &rException, IEvent &event)
{
  IString str ("Encountered an exception:\n\n");

  if (!rException. textCount ())
      str += "No error text available";
  else
      {
       char szBuffer [21];
       str += rException. text (rException. textCount () - 1);
       str += " Error ID: ";
       str += _ltoa (rException. errorId (), szBuffer, 10);
      }

  IMessageBox msgBox (event. window ());

  msgBox. setTitle (rException. name ());

  IMessageBox :: Response result =  msgBox.show(str, IMessageBox::okCancelButton   |
                                                     IMessageBox::informationIcon  |
                                                     IMessageBox::applicationModal |
                                                     IMessageBox::moveable);

  return((result == IMessageBox :: ok ? true : false));
}

